/**
 * asm_FRDMKL25Z.s
 * Pisca-pisca de um led azul
 * Projeto de Controle de Pisca-Pisca do Led em GNU Assembly (GAS)
 * Cdigo utilizado na disciplina EA871 - Segundo semestre de 2020
 * Autora: Wu Shin Ting
 * Data:  19/09/2020
 *
 * Nota 1: Doxygen nao suporta documentacao de codigos em assembly.
 * Nota 2: ARM e Thumb Quick Reference Card: https://documentation-service.arm.com/static/5ed66080ca06a95ce53f932d?token=
 * Nota 3: Sinopse de cada instrucao: ftp://ftp.dca.fee.unicamp.br/pub/docs/ea871/ARM/ARMv6-M.pdf (A6) 
 */
 
 	.equ ITER_NUMB, 114
 	
	.text                             @ monta no segmento de instrucoes
	.align	2						  @ alinha os enderecos em multiplos de 2 bytes 
	.global	main					  @ nome <main> qualificado como global
	.type main function               @ <main> eh uma funcao 

# Estimativa do intervalo de tempo com base no ciclo de instrucoes
# Frequencia do clock do nucleo e barramento do microcontrolador: 20,97152MHz
# Informacoes em:
# ftp://ftp.dca.fee.unicamp.br/pub/docs/ea871/ARM/ARM7TDMITechnicalManual.pdf - Cap. 6
delay:								  @ <delay>
	push 	{r2,r3,r7,lr}          	  @ empilhe  
									  @ empilhados na sequencia lr->r7->r3              
	sub 	sp,sp,#8				  @ atribua a um registrador (destino) a subtracao de um valor imediato do [SP] (fonte)
									  @ SP <- [SP] - 8
	add 	r7,sp,#0				  @ atribua a um registrador (destino) a soma de um valor imediato ao [SP] (fonte)
									  @ R7 <- [SP] + 0
	str 	r0,[r7,#4]				  @ salve na memoria (destino) o conteudo de um registrador deslocado de um valor imediato (fonte) 
									  @ [[R7]+4] <- [R0]
	ldr 	r3,[r7,#4]				  @ carregue no registrador (destino) o conteudo da memria cujo endereco eh o conteudo de um 
									  @ registrador deslocado de um valor imediato (fonte)
									  @ R3 <- [[R7]+4]
	mov		r2, #3					  @ R2 <- 4
	mov		r3, #ITER_NUMB 			  @ R3 <- 115
iteracao:
laco_externo:      
laco_interno:
	sub 	r3,#1					  @ atribua a registrador a subtracao de um valor imediato
									  @ R3 <- [R3]-1
	bne 	laco_interno			  @ condicional NE (nao eh igual a zero) para desvio
									  @ enquanto nao atingir 0, PC <- <iteracao>
	mov 	r3, #ITER_NUMB			  @ R3 <- 115
	sub		r2, #1					  @ R2 <- [R2]-1
	bne		laco_externo
	mov 	r2, #3					  @ R2 <- 3
	mov 	r3, #ITER_NUMB			  @ R3 <- 115
	sub		r0, #1					  @ R0 <- [R0]-1
	bne		iteracao
	
	mov 	sp,r7					  @ mova da fonte para destino 
									  @ SP <- [R7]
	add		sp,sp,#8				  @ SP <- [SP}+8 
	pop 	{r2,r3,r7,pc}			  @ desempilhe
									  @ desempilhar para os registradores na sequencia r3->r7->pc
									   
main:								  @ <main>
	@  SIM_SCGC5 = SIM_SCGC5 | (1<<13);  


	ldr 	r3,SIM_SCGC5 			  @ R3 <- [SIM_SCGC5]
	ldr 	r2,[r3]				      @ R2 <- [R3]
	movs 	r1,#128					  @ mova fonte para destino com atualizacao das flags de condicoes (128=80h)
									  @ R1 <- 128
	lsl 	r1,r1,#6				  @ deslocamento logico para esquerda 
									  @ R1 <- [R1] << 3 
	orr 	r2,r1					  @ OR logico entre fonte e destino
									  @ R2 <- [R2] | [R1]
	str 	r2,[r3]                   @ [R3] <- [R2]
	@  PORTE_PCR23 = PORTE_PCR23 & 0xFFFFF8FF; 
	ldr 	r3,PORTE_PCR23 			  @ R3 <- [PORTE_PCR23]
	ldr 	r1,[r3]				      @ R1 <- [R3]
	ldr 	r2,RESET_MUX			  @ R2 <- [RESET_MUX]
	and 	r2,r1					  @ AND logico entre fonte e destino
									  @ R2 <- [R2] & [R1]
	@  PORTE_PCR23 = PORTE_PCR23 | 0x00000100; 
	movs 	r1,#128					  @ R1 <- 128
	lsl 	r1,r1,#1			      @ R1 < [R1] << 1
	orr 	r2,r1					  @ R2 <- [R2] | [R1]	
	str 	r2,[r3]			    	  @ [R3] <- R2
	@  GPIOE_PDDR  = GPIOE_PDDR  | (1<<23);   
	ldr 	r3,GPIOE_PDDR 			  @ R3 <- [GPIOE_PDDR]
	ldr 	r2,[r3]		    		  @ R2 <- [R3]
	movs 	r1,#128					  @ R1 <- 128
	lsl 	r1,r1,#16				  @ R1 < [R1] << 16
	orr 	r2,r1					  @ R2 <- [R2] | [R1]	
	str 	r2,[r3]			    	  @ [R3] <- R2
loop:								  
	@  GPIOE_PSOR = (1<<23); 
	ldr 	r3,GPIOE_PSOR 			  @ R3 <- [GPIOE_PSOR]
	movs 	r2,#128					  @ R2 <- 128
	lsl 	r2,r2,#16				  @ R2 <- [R2] << 16
	str 	r2,[r3]			     	  @ [R3] <- R2
	@  delay(10000);       
	ldr 	r0,COUNT				  @ R0 <- [COUNT]
	bl 	delay          				  @ salve [PC+2] e desvie
									  @ PC <- <delay> 
	@  GPIOE_PCOR = (1<<23); 
	ldr 	r3,GPIOE_PCOR 			  @ R3 <- [GPIOE_PCOR]
	movs 	r2,#128					  @ R2 <- 128
	lsl 	r2,r2,#16				  @ R2 <- [R2] << 16
	str 	r2,[r3]			    	  @ [R3] <- R2
	@  delay(10000);       
	ldr 	r0,COUNT				  @ R0 <- [COUNT]
	bl 	delay 						  @ PC <- <delay>		
	@  retorna para incio do lao
	b 	loop						  @ desvie para endereco
									  @ PC <- <loop>

	.align 4							@ alinha os endereco em multiplos de 4 bytes
SIM_SCGC5: @ Endereo do SIM_SCGC5
	.word 0x40048038
PORTE_PCR23: @ Endereo do PORTE_PCR23
	.word 0x4004D05C
GPIOE_PDDR: @ Endereo do GPIOE_PDDR
	.word 0x400FF114
GPIOE_PSOR: @ Endereo do GPIOE_PSOR
	.word 0x400FF104
GPIOE_PCOR: @Endereo do GPIOE_PCOR
	.word 0x400FF108
	
RESET_MUX:
	.word 0xFFFFF8FF
COUNT:
	.word 10000
	
	.end
